unit FrmSplash;

{

   COPYRIGHT 2017 .. 2018 Blue Hell / Jan Punter

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation;

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  For all listed email addresses :

    _dot. to be substituted by a dot      '.'
    2@t2  to be substituted by an at sign '@'


  Blue Hell is a trade mark owned by

    Jan Punter
    https://www.bluehell.nl/
    jan2@t2mail_dot_bluehell_dot_nl
}

interface

uses

  Winapi.Windows, Winapi.Messages,

  System.SysUtils, System.Variants, System.Classes,

  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls,

  KnobsWorms, knobs2013, IntSequences, Vcl.Imaging.pngimage;


type

  TFormSplash = class(TForm)
    KnobsGridControl: TKnobsGridControl;
    LabelMain: TLabel;
    Image: TImage;
    Timer: TTimer;
    Label0: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTimer(Sender: TObject);  private
  private
    FGameOfLife : TGameOfLife;
  public
    procedure   InitLife;
    procedure   DoneLife;
    procedure   StepLife;
    procedure   ShutDown;
    procedure   SetMsg( const aMsg: string);
  end;

var

  FormSplash: TFormSplash;

implementation


{$R *.dfm}

//  public

    procedure   TFormSplash.InitLife;
    begin
      if DebugHook = 0
      then FormStyle := fsStayOnTop;

      KnobsGridControl.FillRandom( 0.35);
      FGameOfLife := TGameOfLife.Create( KnobsGridControl.DataWidth, KnobsGridControl.DataHeight);
      FGameOfLife .Wrapped  := True;
      FGameOfLife.FaultRate := 0.0005;
      FGameOfLife.Data      := KnobsGridControl.GetData;
    end;


    procedure   TFormSplash.DoneLife;
    begin
      Timer.Enabled := False;
      FreeAndNil( FGameOfLife);
    end;


    procedure   TFormSplash.StepLife;
    begin
      FGameOfLife.Execute;

      if FGameOfLife.FillCount < 5
      then begin
        KnobsGridControl.FillRandom( 0.05);
        FGameOfLife.Data := KnobsGridControl.GetData;
      end;

      KnobsGridControl.SetData( FGameOfLife.Data);
      KnobsGridControl.Update;
    end;


    procedure   TFormSplash.ShutDown;
    begin
      SetMsg( 'Started');
      Timer.Enabled := True;
    end;


    procedure   TFormSplash.SetMsg( const aMsg: string);
    begin
      LabelMain.Caption := aMsg;
      Label0   .Caption := aMsg;
      Label1   .Caption := aMsg;
      Label2   .Caption := aMsg;
      Label3   .Caption := aMsg;
    end;


procedure TFormSplash.TimerTimer(Sender: TObject);
begin
  if AlphaBlendValue >= 96
  then begin
    AlphaBlendValue := AlphaBlendValue - 24;
    StepLife;
  end
  else begin
    Timer.Enabled := False;
    FreeAndNil( FormSplash);
  end;
end;

procedure TFormSplash.FormCreate(Sender: TObject);
begin
  InitLife;
end;

procedure TFormSplash.FormDestroy(Sender: TObject);
begin
  DoneLife;
end;

end.

