unit FrmViewer;

{

   COPYRIGHT 2017 .. 2018 Blue Hell / Jan Punter

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation;

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  For all listed email addresses :

    _dot. to be substituted by a dot      '.'
    2@t2  to be substituted by an at sign '@'


  Blue Hell is a trade mark owned by

    Jan Punter
    https://www.bluehell.nl/
    jan2@t2mail_dot_bluehell_dot_nl
}


interface

uses

  Winapi.Windows, Winapi.Messages,

  System.SysUtils,

  System.Variants, System.Classes,

  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.Buttons, Vcl.Samples.Spin,

  knobs2013, KnobsUtils, ComplexMath;


type

  TOnFrameClose = procedure( const aSender: TObject; anId: Integer) of object;

  TFrameViewer = class(TFrame)
    PanelTop: TPanel;
    Viewer: TKnobsDataMaker;
    datagraph_mode: TKnobsSelector;
    CheckBoxLockValue: TCheckBox;
    CheckBoxLockTime: TCheckBox;
    CheckBoxLoopable: TCheckBox;
    SpeedButtonRecord: TKnobsTimerSpeedButton;
    LabelParamName: TLabel;
    KnobsTimerSpeedButtonClose: TKnobsTimerSpeedButton;
    procedure ViewerPointChanged(const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
    procedure KnobsTimerSpeedButtonCloseClick(Sender: TObject);
    procedure CheckBoxLoopableClick(Sender: TObject);
    procedure CheckBoxLockValueClick(Sender: TObject);
    procedure CheckBoxLockTimeClick(Sender: TObject);
    procedure SpeedButtonRecordClick(Sender: TObject);
    procedure datagraph_modeValueChanged(const aSender: TObject; const aPath, aControlType: string; aValue: Double;
      IsFinal, IsAutomation: Boolean);
    procedure ViewerPointAdded(const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
    procedure ViewerPointRemoved(const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
  private
    FId             : Integer;
    FControl        : TKnobsValuedControl;
    FCursorX        : Integer;
    FCursorY        : Integer;
    FGraphMode      : TKnobsDMGraphMode;
    FAllowXMoves    : Boolean;
    FAllowYMoves    : Boolean;
    FViewTimed      : Boolean;
    FLoopable       : Boolean;
    FParamName      : string;
    FRecording      : Boolean;
    FOnPointChanged : TKnobsOnPointChanged;
    FOnPointAdded   : TKnobsOnPointChanged;
    FOnPointRemoved   : TKnobsOnPointChanged;
    FOnFrameClose   : TOnFrameClose;
  private
    procedure   SetCursorX    ( aValue: Integer);
    procedure   SetCursorY    ( aValue: Integer);
    procedure   SetGraphMode  ( aValue : TKnobsDMGraphMode);
    procedure   SetAllowXMoves( aValue: Boolean);
    procedure   SetAllowYMoves( aValue: Boolean);
    procedure   SetViewTimed  ( aValue: Boolean);
    procedure   SetLoopable   ( aValue: Boolean);
    procedure   SetParamName  ( const aValue: string);
    procedure   SetRecording  ( aValue: Boolean);
  private
    procedure   CloseFrame;
    procedure   DoPointChanged( const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
    procedure   DoPointAdded  ( const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
    procedure   DoPointRemoved( const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
  public
    constructor Create( anOwner: TComponent);                                                                  override;
    procedure   Initialize;
    procedure   ClearGraph;
    procedure   SetData( const aValue : TSignalPairArray);
  public
    property    Id             : Integer              read FId             write FId;
    property    Control        : TKnobsValuedControl  read FControl        write FControl;
    property    CursorX        : Integer              read FCursorX        write SetCursorX;
    property    CursorY        : Integer              read FCursorY        write SetCursorY;
    property    GraphMode      : TKnobsDMGraphMode    read FGraphMode      write SetGraphMode;
    property    AllowXMoves    : Boolean              read FAllowXMoves    write SetAllowXMoves;
    property    AllowYMoves    : Boolean              read FAllowYMoves    write SetAllowYMoves;
    property    ViewTimed      : Boolean              read FViewTimed      write SetViewTimed;
    property    Loopable       : Boolean              read FLoopable       write SetLoopable;
    property    ParamName      : string               read FParamName      write SetParamName;
    property    Recording      : Boolean              read FRecording      write SetRecording;
  public
    property    OnPointChanged : TKnobsOnPointChanged read FOnPointChanged write FOnPointChanged;
    property    OnPointAdded   : TKnobsOnPointChanged read FOnPointAdded   write FOnPointAdded;
    property    OnPointRemoved : TKnobsOnPointChanged read FOnPointRemoved write FOnPointRemoved;
    property    OnFrameClose   : TOnFrameClose        read FOnFrameClose   write FOnFrameClose;
  end;

implementation


// User area


    procedure   TFrameViewer.SetCursorX( aValue: Integer);
    begin
      if   aValue <> FCursorX
      then begin
        FCursorX       := aValue;
        Viewer.CursorX := aValue;
      end;
    end;


    procedure   TFrameViewer.SetCursorY( aValue: Integer);
    begin
      if   aValue <> FCursorY
      then begin
        FCursorY       := aValue;
        Viewer.CursorY := aValue;
      end;
    end;


    procedure   TFrameViewer.SetGraphMode( aValue : TKnobsDMGraphMode);
    begin
      if   aValue <> FGraphMode
      then begin
        FGraphMode       := aValue;
        Viewer.GraphMode := aValue;
        datagraph_mode.KnobPosition := Ord( aValue);
      end;
    end;


    procedure   TFrameViewer.SetAllowXMoves( aValue: Boolean);
    begin
      if   aValue <> FAllowXMoves
      then begin
        FAllowXMoves                 :=     aValue;
        Viewer          .AllowXMoves :=     aValue;
        CheckBoxLockTime.Checked     := not aValue;

        if   not aValue
        then AllowYMoves := True;
      end;
    end;


    procedure   TFrameViewer.SetAllowYMoves( aValue: Boolean);
    begin
      if   aValue <> FAllowYMoves
      then begin
        FAllowYMoves                  :=     aValue;
        Viewer           .AllowYMoves :=     aValue;
        CheckBoxLockValue.Checked     := not aValue;

        if   not aValue
        then AllowXMoves := True;
      end;
    end;


    procedure   TFrameViewer.SetViewTimed( aValue: Boolean);
    begin
      if   aValue <> FViewTimed
      then begin
        FViewTimed := aValue;
      end;
    end;


    procedure   TFrameViewer.SetLoopable( aValue: Boolean);
    begin
      if aValue <> FLoopable
      then begin
        FLoopable                := aValue;
        Viewer.AutoLoop          := aValue;
        CheckBoxLoopable.Checked := aValue;
      end;
    end;


    procedure   TFrameViewer.SetParamName( const aValue: string);
    begin
      if aValue <> FParamName
      then begin
        FParamName             := aValue;
        LabelParamName.Caption := aValue;
      end;
    end;


    procedure   TFrameViewer.SetRecording( aValue: Boolean);
    begin
      if aValue <> FRecording
      then begin
        FRecording             := aValue;
        SpeedButtonRecord.Down := FRecording;

        if FRecording
        then SpeedButtonRecord.Font.Color := clRed
        else SpeedButtonRecord.Font.Color := clMaroon;
      end;
    end;


//  private

    procedure   TFrameViewer.CloseFrame;
    begin
      if Assigned( FOnFrameClose)
      then FOnFrameClose( Self, Id);
    end;


    procedure   TFrameViewer.DoPointChanged(const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
    begin
      if Assigned( FOnPointChanged)
      then FOnPointChanged( aSender, aPath, anIndex, anX, anY);
    end;


    procedure   TFrameViewer.DoPointAdded( const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
    begin
      if Assigned( FOnPointAdded)
      then FOnPointAdded( aSender, aPath, anIndex, anX, anY);
    end;


    procedure   TFrameViewer.DoPointRemoved( const aSender: TObject; const aPath: string; anIndex: Integer; anX, anY: Double);
    begin
      if Assigned( FOnPointRemoved)
      then FOnPointRemoved( aSender, aPath, anIndex, anX, anY);
    end;


//   public

    constructor TFrameViewer.Create( anOwner: TComponent); // override;
    begin
      inherited;
      Viewer.GluedLeft  := False;
      Viewer.GluedRight := False;
      Viewer.AutoHGrid  := False;
      Viewer.AutoVGrid  := True;
    end;


   procedure   TFrameViewer.Initialize;
    begin
      FLoopable    := True      ; Loopable    := False;
      FAllowXMoves := False     ; AllowXMoves := True;
      FAllowYMoves := False     ; AllowYMoves := True;
      FGraphMode   := dmgSplines; GraphMode   := dmgSteps;
    end;


    procedure   TFrameViewer.ClearGraph;
    begin
      Viewer.MakeWave( dmsClear);
    end;


    procedure   TFrameViewer.SetData( const aValue : TSignalPairArray);
    begin
      Viewer.Data := aValue;
    end;


// Delpi area

{$R *.dfm}

procedure TFrameViewer.datagraph_modeValueChanged(const aSender: TObject; const aPath, aControlType: string;
  aValue: Double; IsFinal, IsAutomation: Boolean);
begin
  GraphMode := TKnobsDMGraphMode( Round( aValue));
end;

procedure TFrameViewer.SpeedButtonRecordClick(Sender: TObject);
begin
  Recording := SpeedButtonRecord.Down;
end;

procedure TFrameViewer.CheckBoxLockTimeClick(Sender: TObject);
begin
  AllowXMoves := not CheckBoxLockTime.Checked;
end;

procedure TFrameViewer.CheckBoxLockValueClick(Sender: TObject);
begin
  AllowYMoves := not CheckBoxLockValue.Checked;
end;

procedure TFrameViewer.CheckBoxLoopableClick(Sender: TObject);
begin
  Loopable := CheckBoxLoopable.Checked;
end;

procedure TFrameViewer.KnobsTimerSpeedButtonCloseClick(Sender: TObject);
begin
  CloseFrame;
end;

procedure TFrameViewer.ViewerPointAdded(const aSender: TObject; const aPath: string; anIndex: Integer; anX,
  anY: Double);
begin
  DoPointAdded( aSender, aPath, anIndex, anX, anY);
end;

procedure TFrameViewer.ViewerPointChanged(const aSender: TObject; const aPath: string; anIndex: Integer; anX,
  anY: Double);
begin
  DoPointChanged( aSender, aPath, anIndex, anX, anY);
end;

procedure TFrameViewer.ViewerPointRemoved(const aSender: TObject; const aPath: string; anIndex: Integer; anX,
  anY: Double);
begin
  DoPointRemoved( aSender, aPath, anIndex, anX, anY);
end;

end.

